@@ -21,6 +21,7 @@ INDEX_PY = """
 from datasette.app import Datasette
 import json
 import pathlib
+import os
 
 static_mounts = [
     (static, str((pathlib.Path(".") / static).resolve()))
@@ -33,13 +34,16 @@ try:
 except Exception:
     pass
 
+secret = os.environ.get("DATASETTE_SECRET")
+
 app = Datasette(
     [],
     {database_files},
     static_mounts=static_mounts,
     metadata=metadata{extras},
+    secret=secret,
     cors=True,
-    config={settings}
+    settings={settings}
 ).app()
 """.strip()
 
@@ -102,6 +106,10 @@ def add_vercel_options(cmd):
                 help="Vercel project name to use",
                 required=True,
             ),
+            click.option(
+                "--scope",
+                help="Optional Vercel scope (e.g. a team name)",
+            ),
             click.option(
                 "--no-prod",
                 is_flag=True,
@@ -120,7 +128,17 @@ def add_vercel_options(cmd):
             click.option(
                 "--generate-dir",
                 type=click.Path(dir_okay=True, file_okay=False),
-                help="Output generated application files here",
+                help="Output generated application files and stop without deploying",
+            ),
+            click.option(
+                "--generate-vercel-json",
+                is_flag=True,
+                help="Output generated vercel.json file and stop without deploying",
+            ),
+            click.option(
+                "--vercel-json",
+                type=click.File(),
+                help="Custom vercel.json file to use instead of generating one",
             ),
             click.option(
                 "--setting",
@@ -156,12 +174,19 @@ def _publish_vercel(
     about_url,
     token,
     project,
+    scope,
     no_prod,
     debug,
     public,
     generate_dir,
+    generate_vercel_json,
+    vercel_json,
     settings,
 ):
+    if vercel_json and generate_vercel_json:
+        raise click.ClickException(
+            "Cannot use both --vercel-json and --generate-vercel-json"
+        )
     fail_if_publish_binary_not_installed(
         "vercel", "Vercel", "https://vercel.com/download"
     )
@@ -178,6 +203,26 @@ def _publish_vercel(
     if generate_dir:
         generate_dir = str(pathlib.Path(generate_dir).resolve())
 
+    vercel_json_content = json.dumps(
+        {
+            "name": project,
+            "version": 2,
+            "builds": [{"src": "index.py", "use": "@vercel/python"}],
+            "routes": [{"src": "(.*)", "dest": "index.py"}],
+        },
+        indent=4,
+    )
+    if generate_vercel_json:
+        click.echo(vercel_json_content)
+        return
+
+    if vercel_json:
+        vercel_json_content = vercel_json.read()
+        try:
+            json.loads(vercel_json_content)
+        except ValueError:
+            raise click.ClickException("--vercel-json contents must be valid JSON")
+
     with temporary_docker_directory(
         files,
         "datasette-now-v2",
@@ -196,22 +241,12 @@ def _publish_vercel(
     ):
         # We don't actually want the Dockerfile
         os.remove("Dockerfile")
-        open("vercel.json", "w").write(
-            json.dumps(
-                {
-                    "name": project,
-                    "version": 2,
-                    "builds": [{"src": "index.py", "use": "@vercel/python"}],
-                    "routes": [{"src": "(.*)", "dest": "index.py"}],
-                },
-                indent=4,
-            )
-        )
+        open("vercel.json", "w").write(vercel_json_content)
         extras = []
         if template_dir:
-            extras.append('template_dir="{}"'.format(template_dir))
+            extras.append('template_dir="templates"')
         if plugins_dir:
-            extras.append('plugins_dir="{}"'.format(plugins_dir))
+            extras.append('plugins_dir="plugins"')
 
         statics = [item[0] for item in static]
 
@@ -252,6 +287,8 @@ def _publish_vercel(
                 cmd.append("--public")
             if token:
                 cmd.extend(["--token", token])
+            if scope:
+                cmd.extend(["--scope", scope])
             # Add the secret
             cmd.extend(["--env", "DATASETTE_SECRET={}".format(secret)])
             run(cmd)
